@@ -3,13 +3,11 @@ import getpass
 import ConfigParser
 import json
 import sys
-import urlparse
 import webbrowser
 import re
 import os
 from os.path import join, expanduser, abspath, isfile
 
-import requests
 import yaml
 import tutum
 import docker
@@ -18,7 +16,6 @@ from exceptions import StreamOutputError
 from tutum.api import auth
 from tutum.api import exceptions
 from tutumcli import utils
-from . import __version__
 
 
 TUTUM_FILE = '.tutum'
@@ -33,32 +30,6 @@ EXCEPTION_EXIT_CODE = 3
 
 
 def login():
-    def try_register(_username, _password):
-        email = raw_input("Email: ")
-
-        headers = {"Content-Type": "application/json", "User-Agent": "tutum/%s" % __version__}
-        data = {'username': _username, "password1": _password, "password2": _password, "email": email}
-
-        r = requests.post(urlparse.urljoin(tutum.base_url, "register/"), data=json.dumps(data), headers=headers)
-
-        try:
-            if r.status_code == 201:
-                return True, "Account created. Please check your email for activation instructions."
-            elif r.status_code == 429:
-                return False, "Too many retries. Please login again later."
-            else:
-                messages = r.json()['register']
-                if isinstance(messages, dict):
-                    _text = []
-                    for key in messages.keys():
-                        _text.append("%s: %s" % (key, '\n'.join(messages[key])))
-                    _text = '\n'.join(_text)
-                else:
-                    _text = messages
-                return False, _text
-        except Exception:
-            return False, r.text
-
     username = raw_input("Username: ")
     password = getpass.getpass()
     try:
@@ -72,18 +43,19 @@ def login():
                 config.write(cfgfile)
             print("Login succeeded!")
     except exceptions.TutumAuthError:
-        registered, text = try_register(username, password)
+        registered, text = utils.try_register(username, password)
         if registered:
             print(text)
         else:
             if 'username: A user with that username already exists.' in text:
                 print("Wrong username and/or password. Please try to login again", file=sys.stderr)
                 sys.exit(TUTUM_AUTH_ERROR_EXIT_CODE)
-            text = text.replace('password1', 'password')
-            text = text.replace('password2', 'password')
-            text = text.replace('\npassword: This field is required.', '', 1)
-            print(text, file=sys.stderr)
-            sys.exit(TUTUM_AUTH_ERROR_EXIT_CODE)
+            else:
+                text = text.replace('password1', 'password')
+                text = text.replace('password2', 'password')
+                text = text.replace('\npassword: This field is required.', '', 1)
+                print(text, file=sys.stderr)
+                sys.exit(TUTUM_AUTH_ERROR_EXIT_CODE)
     except Exception as e:
         print(e, file=sys.stderr)
         sys.exit(EXCEPTION_EXIT_CODE)
@@ -137,16 +109,12 @@ def build(tag, working_directory, quiet, no_cache):
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def cluster_alias(identifiers, dns):
+def service_inspect(identifiers):
     has_exception = False
     for identifier in identifiers:
         try:
-            cluster_details = utils.fetch_remote_cluster(identifier)
-            if dns is not None:
-                cluster_details.web_public_dns = dns
-                result = cluster_details.save()
-                if result:
-                    print(cluster_details.uuid)
+            service = utils.fetch_remote_service(identifier)
+            print(json.dumps(tutum.Service.fetch(service.uuid).get_all_attributes(), indent=2))
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
@@ -154,65 +122,32 @@ def cluster_alias(identifiers, dns):
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def cluster_inspect(identifiers):
+def service_logs(identifiers):
     has_exception = False
     for identifier in identifiers:
         try:
-            cluster = utils.fetch_remote_cluster(identifier)
-            print(json.dumps(tutum.Cluster.fetch(cluster.uuid).get_all_attributes(), indent=2))
+            service = utils.fetch_remote_service(identifier)
+            print(service.logs)
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
     if has_exception:
         sys.exit(EXCEPTION_EXIT_CODE)
 
-
-def cluster_logs(identifiers):
-    has_exception = False
-    for identifier in identifiers:
-        try:
-            cluster = utils.fetch_remote_cluster(identifier)
-            print(cluster.logs)
-        except Exception as e:
-            print(e, file=sys.stderr)
-            has_exception = True
-    if has_exception:
-        sys.exit(EXCEPTION_EXIT_CODE)
-
-
-def cluster_open():
+def service_ps(quiet=False, status=None):
     try:
-        cluster_list = tutum.Cluster.list()
-        deployed_datetimes = {}
-        for cluster in cluster_list:
-            if cluster.web_public_dns and cluster.state in ["Running", "Partly running"]:
-                deployed_datetimes[
-                    utils.from_utc_string_to_utc_datetime(cluster.deployed_datetime)] = cluster.web_public_dns
-        if deployed_datetimes:
-            max_datetime = max(deployed_datetimes.keys())
-            webbrowser.open("http://" + deployed_datetimes[max_datetime])
-        else:
-            print("Error: There are not web applications Running or Partly Running")
-    except Exception as e:
-        print(e, file=sys.stderr)
-        sys.exit(EXCEPTION_EXIT_CODE)
-
-
-def cluster_ps(quiet=False, status=None):
-    try:
-        headers = ["NAME", "UUID", "STATUS", "IMAGE", "DEPLOYED", "WEB HOSTNAME"]
-        cluster_list = tutum.Cluster.list(state=status)
+        headers = ["NAME", "UUID", "STATUS", "IMAGE", "DEPLOYED"]
+        service_list = tutum.Service.list(state=status)
         data_list = []
         long_uuid_list = []
-        for cluster in cluster_list:
-            data_list.append([cluster.unique_name, cluster.uuid[:8],
-                              utils.add_unicode_symbol_to_state(cluster.state),
-                              cluster.image_name,
-                              utils.get_humanize_local_datetime_from_utc_datetime_string(cluster.deployed_datetime),
-                              cluster.web_public_dns])
-            long_uuid_list.append(cluster.uuid)
+        for service in service_list:
+            data_list.append([service.unique_name, service.uuid[:8],
+                              utils.add_unicode_symbol_to_state(service.state),
+                              service.image_name,
+                              utils.get_humanize_local_datetime_from_utc_datetime_string(service.deployed_datetime)])
+            long_uuid_list.append(service.uuid)
         if len(data_list) == 0:
-            data_list.append(["", "", "", "", "", ""])
+            data_list.append(["", "", "", "", ""])
 
         if quiet:
             for uuid in long_uuid_list:
@@ -225,14 +160,14 @@ def cluster_ps(quiet=False, status=None):
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def cluster_redeploy(identifiers, tag):
+def service_redeploy(identifiers, tag):
     has_exception = False
     for identifier in identifiers:
         try:
-            cluster = utils.fetch_remote_cluster(identifier)
-            result = cluster.redeploy(tag)
+            service = utils.fetch_remote_service(identifier)
+            result = service.redeploy(tag)
             if result:
-                print(cluster.uuid)
+                print(service.uuid)
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
@@ -240,39 +175,39 @@ def cluster_redeploy(identifiers, tag):
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def cluster_run(image, name, cpu_shares, memory, memory_swap, target_num_containers, run_command, entrypoint,
-                container_ports, container_envvars, linked_to_cluster, linked_to_container, autorestart, autoreplace,
-                autodestroy, roles, sequential, web_public_dns):
+def service_run(image, name, cpu_shares, memory, memory_swap, target_num_containers, run_command, entrypoint,
+                container_ports, container_envvars, linked_to_service, linked_to_container, autorestart, autoreplace,
+                autodestroy, roles, sequential):
     try:
         ports = utils.parse_ports(container_ports)
         envvars = utils.parse_envvars(container_envvars)
-        links_cluster = utils.parse_links(linked_to_cluster, 'to_cluster')
+        links_service = utils.parse_links(linked_to_service, 'to_service')
         links_container = utils.parse_links(linked_to_container, 'to_container')
-        cluster = tutum.Cluster.create(image=image, name=name, cpu_shares=cpu_shares,
+        service = tutum.Service.create(image=image, name=name, cpu_shares=cpu_shares,
                                        memory=memory, memory_swap=memory_swap,
                                        target_num_containers=target_num_containers, run_command=run_command,
                                        entrypoint=entrypoint, container_ports=ports, container_envvars=envvars,
-                                       linked_to_cluster=links_cluster, linked_to_container=links_container,
+                                       linked_to_service=links_service, linked_to_container=links_container,
                                        autorestart=autorestart, autoreplace=autoreplace, autodestroy=autodestroy,
-                                       roles=roles, sequential_deployment=sequential, web_public_dns=web_public_dns)
-        cluster.save()
-        result = cluster.start()
+                                       roles=roles, sequential_deployment=sequential)
+        service.save()
+        result = service.start()
         if result:
-            print(cluster.uuid)
+            print(service.uuid)
     except Exception as e:
         print(e, file=sys.stderr)
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def cluster_scale(identifiers, target_num_containers):
+def service_scale(identifiers, target_num_containers):
     has_exception = False
     for identifier in identifiers:
         try:
-            cluster = utils.fetch_remote_cluster(identifier)
-            cluster.target_num_containers = target_num_containers
-            result = cluster.save()
+            service = utils.fetch_remote_service(identifier)
+            service.target_num_containers = target_num_containers
+            result = service.save()
             if result:
-                print(cluster.uuid)
+                print(service.uuid)
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
@@ -280,18 +215,18 @@ def cluster_scale(identifiers, target_num_containers):
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def cluster_set(autorestart, autoreplace, autodestroy, identifiers):
+def service_set(autorestart, autoreplace, autodestroy, identifiers):
     has_exception = False
     for identifier in identifiers:
         try:
-            cluster_details = utils.fetch_remote_cluster(identifier, raise_exceptions=True)
-            if cluster_details is not None:
-                cluster_details.autorestart = autorestart
-                cluster_details.autoreplace = autoreplace
-                cluster_details.autodestroy = autodestroy
-                result = cluster_details.save()
+            service_details = utils.fetch_remote_service(identifier, raise_exceptions=True)
+            if service_details is not None:
+                service_details.autorestart = autorestart
+                service_details.autoreplace = autoreplace
+                service_details.autodestroy = autodestroy
+                result = service_details.save()
                 if result:
-                    print(cluster_details.uuid)
+                    print(service_details.uuid)
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
@@ -299,14 +234,14 @@ def cluster_set(autorestart, autoreplace, autodestroy, identifiers):
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def cluster_start(identifiers):
+def service_start(identifiers):
     has_exception = False
     for identifier in identifiers:
         try:
-            cluster = utils.fetch_remote_cluster(identifier)
-            result = cluster.start()
+            service = utils.fetch_remote_service(identifier)
+            result = service.start()
             if result:
-                print(cluster.uuid)
+                print(service.uuid)
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
@@ -314,14 +249,14 @@ def cluster_start(identifiers):
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def cluster_stop(identifiers):
+def service_stop(identifiers):
     has_exception = False
     for identifier in identifiers:
         try:
-            cluster = utils.fetch_remote_cluster(identifier)
-            result = cluster.stop()
+            service = utils.fetch_remote_service(identifier)
+            result = service.stop()
             if result:
-                print(cluster.uuid)
+                print(service.uuid)
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
@@ -329,14 +264,14 @@ def cluster_stop(identifiers):
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def cluster_terminate(identifiers):
+def service_terminate(identifiers):
     has_exception = False
     for identifier in identifiers:
         try:
-            cluster = utils.fetch_remote_cluster(identifier)
-            result = cluster.delete()
+            service = utils.fetch_remote_service(identifier)
+            result = service.delete()
             if result:
-                print(cluster.uuid)
+                print(service.uuid)
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
@@ -370,17 +305,17 @@ def container_logs(identifiers):
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def container_ps(cluster_identifier, quiet=False, status=None):
+def container_ps(identifier, quiet=False, status=None):
     try:
         headers = ["NAME", "UUID", "STATUS", "IMAGE", "RUN COMMAND", "EXIT CODE", "DEPLOYED", "PORTS"]
 
-        if cluster_identifier is None:
+        if identifier is None:
             containers = tutum.Container.list(state=status)
-        elif utils.is_uuid4(cluster_identifier):
-            containers = tutum.Container.list(cluster__uuid=cluster_identifier, state=status)
+        elif utils.is_uuid4(identifier):
+            containers = tutum.Container.list(uuid=identifier, state=status)
         else:
-            containers = tutum.Container.list(cluster__unique_name=cluster_identifier, state=status) + \
-                         tutum.Container.list(cluster__uuid__startswith=cluster_identifier, state=status)
+            containers = tutum.Container.list(unique_name=identifier, state=status) + \
+                         tutum.Container.list(uuid__startswith=identifier, state=status)
 
         data_list = []
         long_uuid_list = []
@@ -416,45 +351,6 @@ def container_ps(cluster_identifier, quiet=False, status=None):
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def container_redeploy(identifiers, tag):
-    has_exception = False
-    for identifier in identifiers:
-        try:
-            container = utils.fetch_remote_container(identifier)
-            result = container.redeploy(tag)
-            if result:
-                print(container.uuid)
-        except Exception as e:
-            print(e, file=sys.stderr)
-            has_exception = True
-    if has_exception:
-        sys.exit(EXCEPTION_EXIT_CODE)
-
-
-def container_run(image, name, cpu_shares, memory, memory_swap, run_command, entrypoint, container_ports,
-                  container_envvars, linked_to_cluster, linked_to_container, autorestart, autoreplace, autodestroy,
-                  roles, web_public_dns):
-    try:
-        ports = utils.parse_ports(container_ports)
-        envvars = utils.parse_envvars(container_envvars)
-        links_cluster = utils.parse_links(linked_to_cluster, 'to_cluster')
-        links_container = utils.parse_links(linked_to_container, 'to_container')
-        container = tutum.Container.create(image=image, name=name, cpu_shares=cpu_shares,
-                                           memory=memory, memory_swap=memory_swap,
-                                           run_command=run_command,
-                                           entrypoint=entrypoint, container_ports=ports, container_envvars=envvars,
-                                           linked_to_cluster=links_cluster, linked_to_container=links_container,
-                                           autorestart=autorestart, autoreplace=autoreplace, autodestroy=autodestroy,
-                                           roles=roles, web_public_dns=web_public_dns)
-        container.save()
-        result = container.start()
-        if result:
-            print(container.uuid)
-    except Exception as e:
-        print(e, file=sys.stderr)
-        sys.exit(EXCEPTION_EXIT_CODE)
-
-
 def container_start(identifiers):
     has_exception = False
     for identifier in identifiers:
@@ -502,7 +398,6 @@ def container_terminate(identifiers):
 
 def image_list(quiet=False, jumpstarts=False, linux=False):
     try:
-
         headers = ["NAME", "DESCRIPTION"]
         data_list = []
         name_list = []
@@ -668,6 +563,7 @@ def image_search(text):
 
 
 def image_update(repositories, username, password, description):
+    has_exception = False
     for repository in repositories:
         try:
             image = tutum.Image.fetch(repository)
@@ -682,9 +578,12 @@ def image_update(repositories, username, password, description):
                 print(image.name)
         except Exception as e:
             print(e, file=sys.stderr)
+            has_exception = True
+    if has_exception:
+        sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def node_list(quiet):
+def node_list(quiet=False):
     try:
         headers = ["UUID", "FQDN", "LASTSEEN", "STATUS", "CLUSTER"]
         node_list = tutum.Node.list()
@@ -700,7 +599,7 @@ def node_list(quiet):
             data_list.append([node.uuid[:8],
                               node.external_fqdn,
                               utils.get_humanize_local_datetime_from_utc_datetime_string(node.last_seen),
-                              node.state,
+                              utils.add_unicode_symbol_to_state(node.state),
                               cluster_name])
             long_uuid_list.append(node.uuid)
         if len(data_list) == 0:
@@ -757,9 +656,9 @@ def nodecluster_list(quiet):
             node_type = nodecluster.node_type
             region = nodecluster.region
             try:
-                node_type = tutum.NodeType.fetch(nodecluster.node_type.strip("/").split("/")[-1]).label
-                region = tutum.Region.fetch(nodecluster.region.strip("/").split("/")[-1]).label
-            except:
+                node_type = tutum.NodeType.fetch(nodecluster.node_type.strip("/").split("api/v1/nodetype/")[-1]).label
+                region = tutum.Region.fetch(nodecluster.region.strip("/").split("api/v1/region/")[-1]).label
+            except Exception as e:
                 pass
 
             data_list.append([nodecluster.name,
@@ -798,7 +697,7 @@ def nodecluster_inspect(identifiers):
 
 def nodecluster_show_providers(quiet):
     try:
-        headers = ["ID", "NAME", "REGIONS"]
+        headers = ["NAME", "LABEL", "REGIONS"]
         data_list = []
         name_list = []
         provider_list = tutum.Provider.list()
@@ -807,14 +706,8 @@ def nodecluster_show_providers(quiet):
                 name_list.append(provider.name)
                 continue
 
-            region_label = []
-            for region in provider.regions:
-                try:
-                    region_obj = tutum.Region.fetch(region.strip("/").split("/")[-1])
-                    region_label.append(region_obj.label or region_obj.name)
-                except:
-                    pass
-            data_list.append([provider.id, provider.name, ", ".join(region_label)])
+            data_list.append([provider.name, provider.label,
+                              ", ".join([region.strip("/").split("/")[-1] for region in provider.regions])])
 
         if len(data_list) == 0:
             data_list.append(["", "", ""])
@@ -828,74 +721,57 @@ def nodecluster_show_providers(quiet):
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def nodecluster_show_regions(provider_id):
+def nodecluster_show_regions(provider):
     try:
-        provider = tutum.Provider.fetch(str(provider_id))
-        provider_name = provider.name
-    except Exception as e:
-        if "Status 404" in e.message:
-            print("The id(%d) of provider does not exist!" % provider_id, file=sys.stderr)
-            sys.exit(EXCEPTION_EXIT_CODE)
-        print(e, file=sys.stderr)
-        sys.exit(EXCEPTION_EXIT_CODE)
-    print('Listing all available regions of provider: %s ...' % provider_name)
-
-    try:
-        headers = ["ID", "NAME", "TYPE"]
+        headers = ["NAME", "LABEL", "PROVIDER", "TYPE"]
         data_list = []
-        region_list = tutum.Region.list(provider=provider_id)
+        region_list = tutum.Region.list()
         for region in region_list:
-            node_type_list = []
-            for node_type in region.node_types:
-                try:
-                    obj = tutum.NodeType.fetch(node_type.strip("/").split("/")[-1])
-                    node_type_list.append(obj.label or obj.name)
-                except:
-                    pass
-            data_list.append([region.id, region.label or region.name, ", ".join(node_type_list)])
+            provider_name = region.resource_uri.strip("/").split("/")[-2]
+            if provider and provider != provider_name:
+                continue
+            data_list.append([region.name, region.label, provider_name,
+                              ", ".join([nodetype.strip("/").split("/")[-1] for nodetype in region.node_types])])
 
         if len(data_list) == 0:
-            data_list.append(["", "", ""])
+            data_list.append(["", "", "", ""])
         utils.tabulate_result(data_list, headers)
     except Exception as e:
         print(e, file=sys.stderr)
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def nodecluster_show_types(region_id):
+def nodecluster_show_types(provider, region):
     try:
-        region = tutum.Region.fetch(str(region_id))
-        region_name = region.label or region.name
-    except Exception as e:
-        if "Status 404" in e.message:
-            print("The ID(%d) of region does not exist! " % region_id, file=sys.stderr)
-            sys.exit(EXCEPTION_EXIT_CODE)
-        print(e, file=sys.stderr)
-        sys.exit(EXCEPTION_EXIT_CODE)
-    print('Listing all available types of region: %s ...' % region_name)
-
-    try:
-        headers = ["ID", "TYPE"]
+        headers = ["NAME", "LABEL", "PROVIDER", "REGIONS"]
         data_list = []
-        type_list = tutum.NodeType.list(region=region_id)
-        for node_type in type_list:
-            data_list.append([node_type.resource_uri.strip("/").split("/")[-1], node_type.label or node_type.name])
+        nodetype_list = tutum.NodeType.list()
+        for nodetype in nodetype_list:
+            provider_name = nodetype.resource_uri.strip("/").split("/")[-2]
+            regions = [region_uri.strip("/").split("/")[-1] for region_uri in nodetype.regions]
+            if provider and provider != provider_name:
+                continue
+
+            if region and region not in regions:
+                continue
+            data_list.append([nodetype.name, nodetype.label, provider_name,
+                              ", ".join(regions)])
+
         if len(data_list) == 0:
-            data_list.append(["", "", ""])
+            data_list.append(["", "", "", ""])
         utils.tabulate_result(data_list, headers)
     except Exception as e:
         print(e, file=sys.stderr)
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def nodecluster_create(target_num_nodes, name, provider_id, region_id, nodetype_id):
-    provider = "/api/v1/provider/%d/" % provider_id
-    region = "/api/v1/region/%d/" % region_id
-    nodetype = "/api/v1/nodetype/%d/" % nodetype_id
+def nodecluster_create(target_num_nodes, name, provider, region, nodetype):
+    region_uri = "/api/v1/region/%s/%s/" % (provider, region)
+    nodetype_uri = "/api/v1/nodetype/%s/%s/" % (provider, nodetype)
 
     try:
         nodecluster = tutum.NodeCluster.create(name=name, target_num_nodes=target_num_nodes,
-                                               provider=provider, region=region, node_type=nodetype)
+                                               region=region_uri, node_type=nodetype_uri)
         nodecluster.save()
         result = nodecluster.deploy()
         if result:
@@ -920,13 +796,12 @@ def nodecluster_rm(identifiers):
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-
-def nodecluster_scale(identifiers, target_num_containers):
+def nodecluster_scale(identifiers, target_num_nodes):
     has_exception = False
     for identifier in identifiers:
         try:
             nodecluster = utils.fetch_remote_nodecluster(identifier)
-            nodecluster.target_num_nodes = target_num_containers
+            nodecluster.target_num_nodes = target_num_nodes
             result = nodecluster.save()
             if result:
                 print(nodecluster.uuid)
@@ -934,4 +809,4 @@ def nodecluster_scale(identifiers, target_num_containers):
             print(e, file=sys.stderr)
             has_exception = True
     if has_exception:
-        sys.exit(EXCEPTION_EXIT_CODE)
+        sys.exit(EXCEPTION_EXIT_CODE)
\ No newline at end of file
